/****************************************************************************
* Simon for Nintendo Wii / Gamecube
* Version: 0.2
*
* Author:  teknecal
* Created: 30/05/2008
* Last Modified: 05/06/2008
* 
****************************************************************************/
#include <gccore.h>
#include <gcmodplay.h> 
#include <ogcsys.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include <time.h>
#include <ogc/pad.h>
#include <ogc/system.h>
#include <png/pngogc.h>

#include "main.h"

// Graphics
extern char toucan[];
extern int toucanlength;

// 2D Video Globals
static u32 *xfb[2] = { NULL, NULL };
static GXRModeObj *vmode;


void Initialise() {
  
	VIDEO_Init();
    PAD_Init();
    
    switch(VIDEO_GetCurrentTvMode()) {
        case VI_NTSC:
            vmode = &TVNtsc480IntDf;
            break;
        case VI_PAL:
            vmode = &TVPal528IntDf;
            break;
        case VI_MPAL:
            vmode = &TVMpal480IntDf;
            break;
        default:
            vmode = &TVNtsc480IntDf;
            break;
    }
	
	VIDEO_Configure(vmode);
	
	xfb[0] = (u32 *) MEM_K0_TO_K1(SYS_AllocateFramebuffer(vmode));
    xfb[1] = (u32 *) MEM_K0_TO_K1(SYS_AllocateFramebuffer(vmode));
	
    console_init(xfb[0], 20, 20, vmode->fbWidth, vmode->xfbHeight,vmode->fbWidth * 2);
    
	VIDEO_ClearFrameBuffer(vmode, xfb[0], COLOR_BLACK);
    VIDEO_ClearFrameBuffer(vmode, xfb[1], COLOR_BLACK);
	
	VIDEO_SetNextFramebuffer(xfb[0]);
    VIDEO_SetBlack(FALSE);
    VIDEO_Flush();
    VIDEO_WaitVSync();
    if(vmode->viTVMode&VI_NON_INTERLACE) VIDEO_WaitVSync();

}

void SetScreen() {
	VIDEO_SetNextFramebuffer(xfb[0]);
	VIDEO_Flush();
	VIDEO_WaitVSync();
}

void clearscreen() {
	int x,y;
	for(y=0;y<486;y++)
		for(x=0;x<320;x++)
			xfb[0][x+320*y]=COLOR_BLACK;
}


int main () {

	
	/*** User defined types from pngogc ***/
  PNGWRAPPER png;
  PNG_MEMFILE *mf;

  /*** General ***/
  char *pic;		/*** Picture buffer ***/
  int row, col;
  u32 *upic;
 
	Initialise();
	SYS_SetResetCallback (reset_cb);
	
	/*** Show some useful information ***/
  printf("Compiled with libpng %s, using libpng %s\n",
         PNG_LIBPNG_VER_STRING, png_libpng_ver);
  printf("Compiled with zlib %s, using zlib %s\n",
         ZLIB_VERSION, zlib_version);
		 
	
	if ( !PNG_IsPNG(toucan) )
    {
      printf("Not a PNG file!\n");
      while(1);
    }

	/*** Create a memory filehandle ***/
  mf = pngmem_fopen(toucan, toucanlength);
  if ( mf == NULL )
    {
      printf("No memory file!\n");
      while(1);
    }

	 /***
   * pngogc helper function to decode the entire image.
   */
  if ( PNG_Decode(mf, &png) == PNGOGC_SUCCESS )
    {

      /***
       * PNGWRAPPER png returns some useful information about the image.
       *
       * Use width and height for picture buffer allocation.
       *
       * The decoded image is held in png.image as 8 bit RGB triples.
       */
      pic = (char *)malloc(png.height * png.width * 2);
      upic = (u32 *)pic;

      /***
       * pngogc helper function RGB24toYUY2
       *
       * This function converts an RGB24 image to the NGC YUY2 format
       */
      RGB24toYUY2(png.image, png.width, png.height, pic, png.width * 2);

      /*** Centre the image on screen ***/
      int offset = (( vmode->xfbHeight - png.height ) >> 1 ) * 320;
      offset += ( vmode->fbWidth - png.width ) >> 2;

      /*** Draw the image to the current framebuffer ***/
      /*int j = 0;
      for ( row = 0; row < png.height; row ++ )
        {
          for( col = 0; col < png.width >> 1; col++ )
            xfb[0][offset+col] = upic[j++];

          offset += 320;
        }
		*/
		
		int offsetx = 0;
		int offsety = 0;

	int i,j;
	for(i=0;i<=(png.width-(png.width/2)-1);i++)
		for(j=1;j<=png.height-2;j++) 
			xfb[0][(i+offsetx)+320*(j+16+offsety)]=upic[i+(png.width-(png.width/2))*j];


      free(pic);

      /***
       * pngogc helper function PNG_Clear
       *
       * This function must be called to release any internal
       * libpng memory and of course, the main RGB24 image data
       */
      PNG_Clear(&png);
      pngmem_fclose(mf);

      /*** Now try to write one ***/
      //pic = (char// *)malloc(vmode->fbWidth * vmode->xfbHeight * 3); /*** Way too big, better to be safe though ***/
      //mf = pngmem_fopen(pic, vmode->fbWidth * vmode->xfbHeight * 3);

      /***
       * pngogc helper function YUY2toRGB24
       *
       * This takes a snapshot of the current framebuffer
       */
      YUY2toRGB24(vmode, pic, (u8 *)((u32)xfb[0] & 0x8fffffff) );

      /***
       * pngogc helper function PNG_Write
       *
       * This takes care of writing the compressed image to the memory file
       */
      //if(PNG_Write( pic, vmode->fbWidth, vmode->xfbHeight, mf ) == PNGOGC_SUCCESS)
       // {
          /*** SD Test ***/
          //sd_file *handle;
          //handle = SDCARD_OpenFile("dev0:\\gcscreen.png", "w");
          //SDCARD_WriteFile(handle, pic, mf->currpos);
          //SDCARD_CloseFile(handle);
      //  }

      /*** Close memory file and free picture buffer ***/
      //pngmem_fclose(mf);
      //free(pic);

      printf("Complete\n");
    }
 
	return 0;
}
